export type QueryResponse<T> = QuerySuccess<T> | QueryFailure;
export type QuerySuccess<T> = {
    status: 200;
    body: {
        summary?: string;
        data: T;
    };
};
export type QueryFailure = {
    status: number;
    body: {
        summary?: string;
        error: {
            code: string;
            message?: string;
        };
    };
};
export default class FaunaClient {
    endpoint: string;
    secret: string;
    timeout?: number;
    constructor(opts: {
        endpoint: string;
        secret: string;
        timeout?: number;
    });
    query<T>(query: string, opts?: {
        format?: string;
        typecheck?: boolean;
        secret?: string;
    }): Promise<QueryResponse<T>>;
    /**
     * We have two different clients, 1 for v10 and 1 for v4.  The v4 client requires closing
     * In order to allow commands to just close their client without having to worry about which
     * client they received, adding this noop method here.
     */
    close(): Promise<void>;
}
