"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
class FaunaClient {
    constructor(opts) {
        this.endpoint = opts.endpoint;
        this.secret = opts.secret;
        this.timeout = opts.timeout;
    }
    async query(query, opts) {
        var _a, _b, _c, _d, _e, _f;
        const { format, typecheck, secret } = {
            format: (_a = opts === null || opts === void 0 ? void 0 : opts.format) !== null && _a !== void 0 ? _a : "simple",
            typecheck: (_b = opts === null || opts === void 0 ? void 0 : opts.typecheck) !== null && _b !== void 0 ? _b : undefined,
            secret: (_c = opts === null || opts === void 0 ? void 0 : opts.secret) !== null && _c !== void 0 ? _c : this.secret,
        };
        const url = new URL(this.endpoint);
        url.pathname = "/query/1";
        const res = await (0, node_fetch_1.default)(url, {
            method: "POST",
            headers: {
                authorization: `Bearer ${secret !== null && secret !== void 0 ? secret : this.secret}`,
                "x-fauna-source": "Fauna Shell",
                ...(typecheck !== undefined && { "x-typecheck": typecheck.toString() }),
                ...(format !== undefined && { "x-format": format }),
                ...((_d = (this.timeout && {
                    "x-query-timeout-ms": this.timeout.toString(10),
                })) !== null && _d !== void 0 ? _d : {}),
            },
            body: JSON.stringify({ query }),
        });
        const json = await res.json();
        if (res.status === 200 || res.status === 201) {
            return {
                status: 200,
                body: json,
            };
        }
        else {
            return {
                status: res.status,
                body: {
                    summary: json.summary,
                    error: {
                        code: (_e = json.error) === null || _e === void 0 ? void 0 : _e.code,
                        message: (_f = json.error) === null || _f === void 0 ? void 0 : _f.message,
                    },
                },
            };
        }
    }
    /**
     * We have two different clients, 1 for v10 and 1 for v4.  The v4 client requires closing
     * In order to allow commands to just close their client without having to worry about which
     * client they received, adding this noop method here.
     */
    close() {
        return Promise.resolve();
    }
}
exports.default = FaunaClient;
